/***************************************************************************
 *   Copyright (C) 2004 by Sebastian Garcia                                *
 *   s_garcia@speedy.com.ar                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
 
#include <qpainter.h>
#include <qfile.h>
 
#include <klocale.h>
#include <kfilemetainfo.h>
#include <kdebug.h>

#include <noatun/app.h>
#include <noatun/player.h>

#include "playlistview.h"
#include "grpplaylistitem.h"

PlaylistViewItem::PlaylistViewItem( PlaylistView *parent, const KURL &p_url )
	: QListViewItem( parent, parent->lastItem() ),
		m_url( p_url ),
		m_grpItem( NULL ),
		m_playing( false ) {

	m_grpItem = new GrpPlaylistItem( p_url );

	KFileMetaInfo meta( p_url );

	if( ! meta.isValid() ) {
		setText( 0, p_url.fileName() );
		return;
	}

	if ( meta.item( "Title" ).string().isNull() || meta.item( "Title" ).string().stripWhiteSpace().isEmpty() )
		setText( 0, p_url.fileName() );
	else
		setText( 0, meta.item( "Title" ).string() );
	setText( 1, meta.item( "Album" ).string() );
	setText( 2, meta.item( "Artist" ).string() );
	setText( 3, meta.item( "Length" ).string() );

/*	QStringList lst = meta.supportedKeys();
	for(QStringList::const_iterator it = lst.begin(); it != lst.end(); it++ )
		kdDebug() << "Clave: \"" << *it << "\", Valor: \"" << meta.item( *it ).value().toString() << "\"" << endl;*/

}

PlaylistViewItem::~PlaylistViewItem() {
	delete m_grpItem;
}

void PlaylistViewItem::paintCell( QPainter * p, const QColorGroup & cg, int column, int width, int align ) {

	if ( m_playing ) { 
		QFont f = p->font();
		f.setBold( true );
		p->setFont(f);
	}

	QListViewItem::paintCell( p, cg, column, width, align );
}

void PlaylistViewItem::setPlaying( bool p_playing ) {
	m_playing = p_playing;
}

PlaylistView::PlaylistView( QWidget *parent, const char *name )
 : KListView(parent, name),
   m_playingItem( NULL ),
	 m_writedItem( NULL ) {

	addColumn( i18n( "Title" ) );
	addColumn( i18n( "Album" ) );
	addColumn( i18n( "Artist" ) );
	addColumn( i18n( "Length" ) );

	setSelectionModeExt( Extended );
	setAllColumnsShowFocus( true );
	setSorting( -1 );
	setDragEnabled( true );
	setAcceptDrops( true );
 
	connect( this, SIGNAL( doubleClicked( QListViewItem*, const QPoint&, int ) ), this, SLOT( executed(  QListViewItem*, const QPoint&, int ) ) );
}

PlaylistView::~PlaylistView() {

}

GrpPlaylistItem *PlaylistView::current() const {
	if ( m_playingItem )
		return m_playingItem->item();
	else
		return NULL;
}

void PlaylistView::addItem( const KURL &p_url, bool p_play ) {

	if( ! QFile::exists( p_url.path() ) )
		return;

	QListViewItem *item = new PlaylistViewItem( this, p_url );

	if ( p_play )
		executed( item, QPoint(), 0 );
}

void PlaylistView::addItems( const KURL::List &p_list, bool p_play ) {
	for( KURL::List::const_iterator it = p_list.begin(); it != p_list.end(); it++ )
			addItem( *it, false );
	
	if( p_play )
		executed( firstChild(), QPoint(), 0 );
}

void PlaylistView::deleteSelected() {
	QPtrList<QListViewItem> items( selectedItems() );
	for (QPtrListIterator<QListViewItem> it( items ); it.current(); ++it) {
	
		if ( napp->player()->current() == static_cast<PlaylistViewItem*>( *it )->item() ) {
			QListViewItem *item = ( *it )->nextSibling();
			while ( item && ( selectedItems().find( item ) != -1 ) )
				item = item->nextSibling();
			
			if ( ! item )
				napp->player()->stop();
			else
				executed( item, QPoint(), 0 );
		}
	
		if ( m_playingItem == *it )
			m_playingItem = NULL;
		static_cast<PlaylistViewItem*>( *it )->item()->remove();
		delete *it;
	}
}

void PlaylistView::clearPlaylist() {
	QListViewItem *item = firstChild();

	napp->player()->stop();

	while ( item ) {
		static_cast<PlaylistViewItem*>( firstChild() )->item()->remove();
		item = item->nextSibling();
	}

	clear();

	m_playingItem = NULL;
}

void PlaylistView::selectFirst() {
	QListViewItem *item = firstChild();

	if ( item )
		executed( item, QPoint(), 0 );
}

void PlaylistView::selectLast() {
	QListViewItem *item = lastItem();
	if ( item )
		executed( item, QPoint(), 0 );
}


void PlaylistView::selectNext() {
	if ( ! m_playingItem )
		return;

	if ( ! m_playingItem->nextSibling() ) {
		selectFirst();
		return;
	}

	executed( m_playingItem->nextSibling(), QPoint(), 0 );
}


void PlaylistView::selectPrev() {
	if ( ! m_playingItem )
		return;

	if ( ! m_playingItem->itemAbove() ) {
		selectLast();
		return;
	}

	executed( m_playingItem->itemAbove(), QPoint(), 0 );
}

void PlaylistView::loadPlaylist( const KURL &p_file ) {
	PlaylistSaver::load( p_file );
}

void PlaylistView::savePlaylist( const KURL &p_file ) {
	m_writedItem = NULL;
	PlaylistSaver::save( p_file );
}

void PlaylistView::executed( QListViewItem *p_item, const QPoint&, int ) {
	if ( m_playingItem ) {
		m_playingItem->setPlaying( false );
		repaintItem( m_playingItem );
	}

	m_playingItem = static_cast<PlaylistViewItem*>( p_item );
	m_playingItem->setPlaying( true );
	repaintItem( m_playingItem );

	emit itemExecuted( m_playingItem->item() );
}

void PlaylistView::readItem( const QMap< QString, QString > &p_prop ) {
	addItem( p_prop[ "url" ] );
}

PlaylistItem PlaylistView::writeItem() {

	if ( ! m_writedItem )
		m_writedItem = static_cast< PlaylistViewItem* >( firstChild() );
	else
		m_writedItem = static_cast< PlaylistViewItem* >( m_writedItem->nextSibling() );

	if ( ! m_writedItem )
		return PlaylistItem( NULL );

	return PlaylistItem( m_writedItem->item() );
}

#include "playlistview.moc"
