/***************************************************************************
 *   Copyright (C) 2004 by Sebastian Garcia                                *
 *   s_garcia@speedy.com.ar                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef SELECTOR_H
#define SELECTOR_H

#include <qdir.h>

#include <kmdichildview.h>
#include <klistview.h>
#include <kurl.h>

class KDirOperator;
class KURL;
class KURLComboBox;
class KFileItem;
class KProgress;

class PlaylistView;

class Selector : public KMdiChildView {

	Q_OBJECT

public:
	Selector( const QString&, const QString&, QWidget*, PlaylistView *, const char* = 0 );

	~Selector();

protected:

	void makeFilePatterns();

	PlaylistView *m_playListView;
	QString m_filePatterns;

};

class GrpList;
class GrpListItemBase;
class GrpListItemFile;
class GrpListItemGroup;

#define RTTI_GrpListItemBase  1000
#define RTTI_GrpListItemFile  1001
#define RTTI_GrpListItemGroup 1002

class GrpListItemBase : public QListViewItem {

public:

	typedef QValueList<GrpListItemBase*> List;

	GrpListItemBase( GrpList* );
	GrpListItemBase( GrpList*, GrpListItemGroup* );

	GrpListItemGroup *getParentGroup();
	const GrpListItemGroup *getParentGroup() const;

	virtual KURL::List getFiles() const = 0;
	virtual KURL::List getSelectedFiles() const = 0;

	virtual GrpListItemBase::List getItems() = 0;
	virtual GrpListItemBase::List getSelectedItems() = 0;

	virtual void removeSelected();

	virtual GrpListItemFile *findFile( const KURL& ) = 0;

	int rtti() const;

protected:

	GrpList *m_list;
	GrpListItemGroup *m_parentGroup;

};

inline GrpListItemGroup *GrpListItemBase::getParentGroup() {
	return m_parentGroup;
}

inline const GrpListItemGroup *GrpListItemBase::getParentGroup() const {
	return m_parentGroup;
}

inline int GrpListItemBase::rtti() const {
	return RTTI_GrpListItemBase;
}

class GrpListItemFile : public GrpListItemBase {

public:

	GrpListItemFile( GrpList*, const KURL&, const QMap< QString, QString >& );
	GrpListItemFile( GrpList*, GrpListItemGroup*, const KURL&, const QMap< QString, QString >& );

	virtual KURL::List getFiles() const;
	virtual KURL::List getSelectedFiles() const;

	virtual GrpListItemBase::List getItems();
	virtual GrpListItemBase::List getSelectedItems();

	virtual GrpListItemFile *findFile( const KURL& );

	const QMap< QString, QString > &getProperties() const;

	int rtti() const { return RTTI_GrpListItemFile; }

protected:

	virtual int compare( QListViewItem *, int, bool ) const;

	KURL m_url;
	QMap< QString, QString > m_prop;

};

inline const QMap< QString, QString > &GrpListItemFile::getProperties() const {
	return m_prop;
}

class GrpListItemGroup : public GrpListItemBase {

	friend class GrpList;

public:

	GrpListItemGroup( GrpList*, const QString& );
	GrpListItemGroup( GrpList*, GrpListItemGroup*, const QString& );

	virtual KURL::List getFiles() const;
	virtual KURL::List getSelectedFiles() const;

	virtual GrpListItemBase::List getItems();
	virtual GrpListItemBase::List getSelectedItems();

	GrpListItemGroup *getGroup( const QString& );

	void addFileItem( const KURL&, const QMap< QString, QString >& );
	void removeItem( GrpListItemBase* );
	virtual GrpListItemFile *findFile( const KURL& );

	int rtti() const { return RTTI_GrpListItemGroup; }

protected:

	QString m_groupText;

};

class GrpList : protected KListView {

friend class GrpListItemBase;
friend class GrpListItemFile;
friend class GrpListItemGroup;

	Q_OBJECT

public:

	GrpList( QWidget *, const char * = 0 );

	void setGroups( const QStringList& );

	void setProgressBar( KProgress* );

	KURL::List getFiles() const;
	KURL::List getSelectedFiles() const;

	GrpListItemBase::List getItems();
	GrpListItemBase::List getSelectedItems();

	GrpListItemFile *findFile( const KURL& );

public slots:

	void addFile( const KURL& );
	void addFile( const KURL&, const QMap<QString, QString> & );
	void addFiles( const KURL::List& );
	void removeSelected();

	void refresh();

	void clearList();

	void saveList( const KURL& );
	void readList( const KURL& );

protected:

	void addGroup( const QString& );
	GrpListItemGroup *getGroup( const QString& );

	void addFileItem( const KURL&, const QMap< QString, QString >& );
	void removeItem( GrpListItemBase* );

	QStringList m_groups;

	KProgress *m_progressBar;
};

class GroupedSelector : public Selector {

	Q_OBJECT

public:

	GroupedSelector( const QString&, const QString&, QWidget*, PlaylistView*, const char * = 0 );

	~GroupedSelector();

	void setGroups( const QStringList& );

protected slots:

	void addFiles();
	void addDir();

	void addSelectedFiles();
	void playSelectedFiles();

	void contextMenu( KListView*, QListViewItem*, const QPoint& );
	void doubleClicked( QListViewItem*, const QPoint&, int );

protected:

	KURL::List getFiles( const QDir& );

	GrpList *m_grpList;

};

inline void GroupedSelector::setGroups( const QStringList &p_groups ) {
	m_grpList->setGroups( p_groups );
}

class FileSelector : public Selector {

	Q_OBJECT

public:

	FileSelector( const QString&, const QString&, QWidget*, PlaylistView *, const char* = 0 );

	~FileSelector();

public slots:	

	void setURL( const KURL& );
	void setURL( const QString& );

	void addSelectedFiles();
	void playSelectedFiles();

protected slots:

	void filterChanged( const QString & );
	void urlEntered( const KURL& );

	void fileSelected( const KFileItem * );

private:

	KDirOperator *m_dirOperator;
	KURLComboBox *m_currentDirCombo;

};

#endif
